pub(crate) struct GeneralAutoderef<'db, Ctx, Steps = Vec<(Ty<'db>, AutoderefKind)>> {
ctx: Ctx,
traits: Option<AutoderefTraits>,
state: AutoderefSnapshot<'db, Steps>,
include_raw_pointers: bool,
use_receiver_trait: bool,
span: Span,
}Expand description
Recursively dereference a type, considering both built-in
dereferences (*) and the Deref trait.
Although called Autoderef it can be configured to use the
Receiver trait instead of the Deref trait.
Fields
ctx: Ctxtraits: Option<AutoderefTraits>state: AutoderefSnapshot<'db, Steps>include_raw_pointers: booluse_receiver_trait: boolspan: SpanImplementations
Sourceimpl<'db, Ctx: AutoderefCtx<'db>> GeneralAutoderef<'db, Ctx>
impl<'db, Ctx: AutoderefCtx<'db>> GeneralAutoderef<'db, Ctx>
pub(crate) fn adjust_steps_as_infer_ok( &mut self, ) -> InferOk<'db, Vec<Adjustment>>
Sourceimpl<'a, 'db> GeneralAutoderef<'db, DefaultAutoderefCtx<'a, 'db>, Vec<(Ty<'db>, AutoderefKind)>>
impl<'a, 'db> GeneralAutoderef<'db, DefaultAutoderefCtx<'a, 'db>, Vec<(Ty<'db>, AutoderefKind)>>
Sourceimpl<'a, 'db> GeneralAutoderef<'db, InferenceContextAutoderefCtx<'a, 'db>, Vec<(Ty<'db>, AutoderefKind)>>
impl<'a, 'db> GeneralAutoderef<'db, InferenceContextAutoderefCtx<'a, 'db>, Vec<(Ty<'db>, AutoderefKind)>>
pub(crate) fn new_from_inference_context( ctx: &'a mut InferenceContext<'db>, base_ty: Ty<'db>, span: Span, ) -> Self
pub(crate) fn ctx(&mut self) -> &mut InferenceContext<'db>
Sourceimpl<'a, 'db> GeneralAutoderef<'db, DefaultAutoderefCtx<'a, 'db>, usize>
impl<'a, 'db> GeneralAutoderef<'db, DefaultAutoderefCtx<'a, 'db>, usize>
Sourceimpl<'db, Ctx, Steps> GeneralAutoderef<'db, Ctx, Steps>where
Ctx: AutoderefCtx<'db>,
Steps: TrackAutoderefSteps<'db>,
impl<'db, Ctx, Steps> GeneralAutoderef<'db, Ctx, Steps>where
Ctx: AutoderefCtx<'db>,
Steps: TrackAutoderefSteps<'db>,
fn new_impl(ctx: Ctx, base_ty: Ty<'db>, span: Span) -> Self
fn infcx(&self) -> &InferCtxt<'db>
fn param_env(&self) -> ParamEnv<'db>
fn interner(&self) -> DbInterner<'db>
fn autoderef_traits(&mut self) -> Option<AutoderefTraits>
fn overloaded_deref_ty(&mut self, ty: Ty<'db>) -> Option<Ty<'db>>
Sourcepub(crate) fn final_ty(&self) -> Ty<'db>
pub(crate) fn final_ty(&self) -> Ty<'db>
Returns the final type we ended up with, which may be an unresolved inference variable.
pub(crate) fn step_count(&self) -> usize
pub(crate) fn take_obligations( &mut self, ) -> Vec<Obligation<'db, Predicate<'db>>>
pub(crate) fn steps(&self) -> &Steps
pub(crate) fn reached_recursion_limit(&self) -> bool
Sourcepub(crate) fn include_raw_pointers(self) -> Self
pub(crate) fn include_raw_pointers(self) -> Self
also dereference through raw pointer types
e.g., assuming ptr_to_Foo is the type *const Foo
fcx.autoderef(span, ptr_to_Foo) => [*const Foo]
fcx.autoderef(span, ptr_to_Foo).include_raw_ptrs() => [*const Foo, Foo]
Sourcepub(crate) fn use_receiver_trait(self) -> Self
pub(crate) fn use_receiver_trait(self) -> Self
Use core::ops::Receiver and core::ops::Receiver::Target as
the trait and associated type to iterate, instead of
core::ops::Deref and core::ops::Deref::Target